@@ -28,6 +28,7 @@
 #include "src/phongo_client.h"
 #include "src/phongo_error.h"
 #include "src/phongo_ini.h"
+#include "src/phongo_log.h"
 #include "src/functions_arginfo.h"
 
 ZEND_DECLARE_MODULE_GLOBALS(mongodb)
@@ -55,27 +56,28 @@ zend_object_handlers* phongo_get_std_object_handlers(void)
 /* }}} */
 
 /* {{{ Memory allocation wrappers */
-static void* php_phongo_malloc(size_t num_bytes) /* {{{ */
+static void* php_phongo_malloc(size_t num_bytes)
 {
 	return pemalloc(num_bytes, 1);
-} /* }}} */
+}
 
-static void* php_phongo_calloc(size_t num_members, size_t num_bytes) /* {{{ */
+static void* php_phongo_calloc(size_t num_members, size_t num_bytes)
 {
 	return pecalloc(num_members, num_bytes, 1);
-} /* }}} */
+}
 
-static void* php_phongo_realloc(void* mem, size_t num_bytes) /* {{{ */
+static void* php_phongo_realloc(void* mem, size_t num_bytes)
 {
 	return perealloc(mem, num_bytes, 1);
-} /* }}} */
+}
 
-static void php_phongo_free(void* mem) /* {{{ */
+static void php_phongo_free(void* mem)
 {
 	if (mem) {
 		pefree(mem, 1);
 	}
-} /* }}} */
+}
+/* }}} */
 
 PHP_RINIT_FUNCTION(mongodb) /* {{{ */
 {
@@ -89,6 +91,16 @@ PHP_RINIT_FUNCTION(mongodb) /* {{{ */
 		zend_hash_init(MONGODB_G(request_clients), 0, NULL, php_phongo_pclient_destroy_ptr, 0);
 	}
 
+	/* Initialize HashTable for loggers, which is initialized to NULL in GINIT
+	 * and destroyed and reset to NULL in RSHUTDOWN. Since this HashTable will
+	 * store logger object zvals, we specify ZVAL_PTR_DTOR as its element
+	 * destructor so that any still-registered loggers can be freed in
+	 * RSHUTDOWN. */
+	if (MONGODB_G(loggers) == NULL) {
+		ALLOC_HASHTABLE(MONGODB_G(loggers));
+		zend_hash_init(MONGODB_G(loggers), 0, NULL, ZVAL_PTR_DTOR, 0);
+	}
+
 	/* Initialize HashTable for APM subscribers, which is initialized to NULL in
 	 * GINIT and destroyed and reset to NULL in RSHUTDOWN. Since this HashTable
 	 * will store subscriber object zvals, we specify ZVAL_PTR_DTOR as its
@@ -142,12 +154,12 @@ static zend_class_entry* php_phongo_fetch_internal_class(const char* class_name,
 	return NULL;
 }
 
-static HashTable* php_phongo_std_get_gc(phongo_compat_object_handler_type* object, zval** table, int* n) /* {{{ */
+static HashTable* php_phongo_std_get_gc(phongo_compat_object_handler_type* object, zval** table, int* n)
 {
 	*table = NULL;
 	*n     = 0;
 	return zend_std_get_properties(object);
-} /* }}} */
+}
 
 PHP_MINIT_FUNCTION(mongodb) /* {{{ */
 {
@@ -160,6 +172,16 @@ PHP_MINIT_FUNCTION(mongodb) /* {{{ */
 
 	(void) type; /* We don't care if we are loaded via dl() or extension= */
 
+	/* Start by disabling libmongoc's default log handler, which could write to
+	 * stdout/stderr. The PHP driver's log handler may be assigned below when
+	 * parsing INI options or at a later point during a request when registering
+	 * a logger. */
+	mongoc_log_set_handler(NULL, NULL);
+
+	/* Disable trace logging. This will be enabled in phongo_log_sync_handlers()
+	 * if the "mongodb.debug" INI option is set. */
+	mongoc_log_trace_disable();
+
 	phongo_register_ini_entries(INIT_FUNC_ARGS_PASSTHRU);
 
 	/* Assign our custom vtable to libbson, so all memory allocation in libbson
@@ -271,6 +293,7 @@ PHP_MINIT_FUNCTION(mongodb) /* {{{ */
 	php_phongo_commandfailedevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_commandstartedevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_commandsucceededevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
+	php_phongo_logsubscriber_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_sdamsubscriber_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_serverchangedevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_serverclosedevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
@@ -297,6 +320,16 @@ PHP_MSHUTDOWN_FUNCTION(mongodb) /* {{{ */
 
 PHP_RSHUTDOWN_FUNCTION(mongodb) /* {{{ */
 {
+	/* Destroy HashTable for loggers, which was initialized in RINIT. */
+	if (MONGODB_G(loggers)) {
+		zend_hash_destroy(MONGODB_G(loggers));
+		FREE_HASHTABLE(MONGODB_G(loggers));
+		MONGODB_G(loggers) = NULL;
+	}
+
+	/* TODO: consider calling phongo_log_sync_handler here since logging may no
+	 * longer be enabled. */
+
 	/* Destroy HashTable for APM subscribers, which was initialized in RINIT. */
 	if (MONGODB_G(subscribers)) {
 		zend_hash_destroy(MONGODB_G(subscribers));
@@ -334,8 +367,9 @@ PHP_GSHUTDOWN_FUNCTION(mongodb) /* {{{ */
 	 * encryption settings. */
 	zend_hash_graceful_reverse_destroy(&mongodb_globals->persistent_clients);
 
-	phongo_log_disable(mongodb_globals->debug_fd);
-	mongodb_globals->debug_fd = NULL;
+	/* TODO: Check that logging actually gets disabled. The logger HashTable
+	 * should be empty by this point. */
+	phongo_log_set_stream(NULL);
 
 	/* Decrement the thread counter. If it reaches zero, we can infer that this
 	 * is the last thread, MSHUTDOWN has been called, persistent clients from
@@ -412,10 +446,10 @@ PHP_MINFO_FUNCTION(mongodb) /* {{{ */
 	php_info_print_table_row(2, "libmongoc SASL", "disabled");
 #endif
 
-#ifdef MONGOC_ENABLE_ICU
-	php_info_print_table_row(2, "libmongoc ICU", "enabled");
+#ifdef MONGOC_ENABLE_SRV
+	php_info_print_table_row(2, "libmongoc SRV", "enabled");
 #else
-	php_info_print_table_row(2, "libmongoc ICU", "disabled");
+	php_info_print_table_row(2, "libmongoc SRV", "disabled");
 #endif
 
 #ifdef MONGOC_ENABLE_COMPRESSION
@@ -482,20 +516,6 @@ PHP_MINFO_FUNCTION(mongodb) /* {{{ */
 	phongo_display_ini_entries(ZEND_MODULE_INFO_FUNC_ARGS_PASSTHRU);
 } /* }}} */
 
-/* {{{ Shared function entries for disabling constructors and unserialize() */
-PHP_FUNCTION(MongoDB_disabled___construct) /* {{{ */
-{
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Accessing private constructor");
-} /* }}} */
-
-PHP_FUNCTION(MongoDB_disabled___wakeup) /* {{{ */
-{
-	PHONGO_PARSE_PARAMETERS_NONE();
-
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME, "%s", "MongoDB\\Driver objects cannot be serialized");
-} /* }}} */
-/* }}} */
-
 /* {{{ Module dependencies and module entry */
 static const zend_module_dep mongodb_deps[] = {
 	/* clang-format off */
