@@ -405,8 +405,6 @@ bool phongo_query_init(zval* return_value, zval* filter, zval* options)
 #undef PHONGO_QUERY_OPT_INT64_DEPRECATED
 #undef PHONGO_QUERY_OPT_STRING
 
-PHONGO_DISABLED_WAKEUP(MongoDB_Driver_Query)
-
 /* Constructs a new Query */
 static PHP_METHOD(MongoDB_Driver_Query, __construct)
 {
@@ -414,7 +412,7 @@ static PHP_METHOD(MongoDB_Driver_Query, __construct)
 	zval* options = NULL;
 
 	PHONGO_PARSE_PARAMETERS_START(1, 2)
-	PHONGO_PARAM_ARRAY_OR_OBJECT(filter)
+	Z_PARAM_ARRAY_OR_OBJECT(filter)
 	Z_PARAM_OPTIONAL
 	Z_PARAM_ARRAY_OR_NULL(options)
 	PHONGO_PARSE_PARAMETERS_END();
