@@ -14,14 +14,14 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_BUFFER_PRIVATE_H
 #define MONGOC_BUFFER_PRIVATE_H
 
 #include <bson/bson.h>
 
-#include "mongoc/mongoc-stream.h"
+#include "mongoc-stream.h"
 
 
 BSON_BEGIN_DECLS
@@ -40,36 +40,25 @@ struct _mongoc_buffer_t {
 
 
 void
-_mongoc_buffer_init (mongoc_buffer_t *buffer,
-                     uint8_t *buf,
-                     size_t buflen,
-                     bson_realloc_func realloc_func,
-                     void *realloc_data);
+_mongoc_buffer_init (
+   mongoc_buffer_t *buffer, uint8_t *buf, size_t buflen, bson_realloc_func realloc_func, void *realloc_data);
 
 bool
-_mongoc_buffer_append (mongoc_buffer_t *buffer,
-                       const uint8_t *data,
-                       size_t data_size);
+_mongoc_buffer_append (mongoc_buffer_t *buffer, const uint8_t *data, size_t data_size);
 
 bool
-_mongoc_buffer_append_from_stream (mongoc_buffer_t *buffer,
-                                   mongoc_stream_t *stream,
-                                   size_t size,
-                                   int32_t timeout_msec,
-                                   bson_error_t *error);
+_mongoc_buffer_append_from_stream (
+   mongoc_buffer_t *buffer, mongoc_stream_t *stream, size_t size, int64_t timeout_msec, bson_error_t *error);
 
 ssize_t
 _mongoc_buffer_try_append_from_stream (mongoc_buffer_t *buffer,
                                        mongoc_stream_t *stream,
                                        size_t size,
-                                       int32_t timeout_msec);
+                                       int64_t timeout_msec);
 
 ssize_t
-_mongoc_buffer_fill (mongoc_buffer_t *buffer,
-                     mongoc_stream_t *stream,
-                     size_t min_bytes,
-                     int32_t timeout_msec,
-                     bson_error_t *error);
+_mongoc_buffer_fill (
+   mongoc_buffer_t *buffer, mongoc_stream_t *stream, size_t min_bytes, int64_t timeout_msec, bson_error_t *error);
 
 void
 _mongoc_buffer_destroy (mongoc_buffer_t *buffer);
