@@ -22,6 +22,18 @@
 #ifndef MONGOC_CONFIG_H
 #define MONGOC_CONFIG_H
 
+/* clang-format off */
+
+/*
+ * NOTICE:
+ * If you're about to update this file and add a config flag, make sure to
+ * update:
+ * o The bitfield in mongoc-handshake-private.h
+ * o _mongoc_handshake_get_config_hex_string() in mongoc-handshake.c
+ * o examples/parse_handshake_cfg.py
+ * o test_handshake_platform_config in test-mongoc-handshake.c
+ */
+
 /* MONGOC_USER_SET_CFLAGS is set from config based on what compiler flags were
  * used to compile mongoc */
 #define MONGOC_USER_SET_CFLAGS ""
@@ -57,7 +69,7 @@
  * MONGOC_ENABLE_SSL_SECURE_TRANSPORT is set from configure to determine if we are
  * compiled with Native SSL support on Darwin
  */
-#define MONGOC_ENABLE_SSL_SECURE_TRANSPORT 1
+#define MONGOC_ENABLE_SSL_SECURE_TRANSPORT 0
 
 #if MONGOC_ENABLE_SSL_SECURE_TRANSPORT != 1
 #  undef MONGOC_ENABLE_SSL_SECURE_TRANSPORT
@@ -68,7 +80,7 @@
  * MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO is set from configure to determine if we are
  * compiled with Native Crypto support on Darwin
  */
-#define MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO 1
+#define MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO 0
 
 #if MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO != 1
 #  undef MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO
@@ -90,7 +102,7 @@
  * MONGOC_ENABLE_SSL_OPENSSL is set from configure to determine if we are
  * compiled with OpenSSL support.
  */
-#define MONGOC_ENABLE_SSL_OPENSSL 0
+#define MONGOC_ENABLE_SSL_OPENSSL 1
 
 #if MONGOC_ENABLE_SSL_OPENSSL != 1
 #  undef MONGOC_ENABLE_SSL_OPENSSL
@@ -101,7 +113,7 @@
  * MONGOC_ENABLE_CRYPTO_LIBCRYPTO is set from configure to determine if we are
  * compiled with OpenSSL support.
  */
-#define MONGOC_ENABLE_CRYPTO_LIBCRYPTO 0
+#define MONGOC_ENABLE_CRYPTO_LIBCRYPTO 1
 
 #if MONGOC_ENABLE_CRYPTO_LIBCRYPTO != 1
 #  undef MONGOC_ENABLE_CRYPTO_LIBCRYPTO
@@ -143,7 +155,7 @@
 /*
  * Use ASN1_STRING_get0_data () rather than the deprecated ASN1_STRING_data
  */
-#define MONGOC_HAVE_ASN1_STRING_GET0_DATA 0
+#define MONGOC_HAVE_ASN1_STRING_GET0_DATA 1
 
 #if MONGOC_HAVE_ASN1_STRING_GET0_DATA != 1
 #  undef MONGOC_HAVE_ASN1_STRING_GET0_DATA
@@ -154,7 +166,7 @@
  * MONGOC_ENABLE_SASL is set from configure to determine if we are
  * compiled with SASL support.
  */
-#define MONGOC_ENABLE_SASL 1
+#define MONGOC_ENABLE_SASL 0
 
 #if MONGOC_ENABLE_SASL != 1
 #  undef MONGOC_ENABLE_SASL
@@ -165,7 +177,7 @@
  * MONGOC_ENABLE_SASL_CYRUS is set from configure to determine if we are
  * compiled with Cyrus SASL support.
  */
-#define MONGOC_ENABLE_SASL_CYRUS 1
+#define MONGOC_ENABLE_SASL_CYRUS 0
 
 #if MONGOC_ENABLE_SASL_CYRUS != 1
 #  undef MONGOC_ENABLE_SASL_CYRUS
@@ -187,7 +199,7 @@
  * have SASL and its version is new enough to use sasl_client_done (),
  * which supersedes sasl_done ().
  */
-#define MONGOC_HAVE_SASL_CLIENT_DONE 1
+#define MONGOC_HAVE_SASL_CLIENT_DONE 0
 
 #if MONGOC_HAVE_SASL_CLIENT_DONE != 1
 #  undef MONGOC_HAVE_SASL_CLIENT_DONE
@@ -214,6 +226,10 @@
 #  undef MONGOC_HAVE_SOCKLEN
 #endif
 
+/**
+ * @brief Defined to 0/1 for whether we were configured with ENABLE_SRV
+ */
+#define MONGOC_ENABLE_SRV 1
 
 /*
  * MONGOC_HAVE_DNSAPI is set from configure to determine if we should use the
@@ -241,7 +257,7 @@
  * MONGOC_HAVE_RES_NDESTROY is set from configure to determine if we
  * have BSD / Darwin's res_ndestroy().
  */
-#define MONGOC_HAVE_RES_NDESTROY 1
+#define MONGOC_HAVE_RES_NDESTROY 0
 
 #if MONGOC_HAVE_RES_NDESTROY != 1
 #  undef MONGOC_HAVE_RES_NDESTROY
@@ -252,7 +268,7 @@
  * MONGOC_HAVE_RES_NCLOSE is set from configure to determine if we
  * have Linux's res_nclose().
  */
-#define MONGOC_HAVE_RES_NCLOSE 0
+#define MONGOC_HAVE_RES_NCLOSE 1
 
 #if MONGOC_HAVE_RES_NCLOSE != 1
 #  undef MONGOC_HAVE_RES_NCLOSE
@@ -313,7 +329,7 @@
  * Set if we have zstd compression support
  *
  */
-#define MONGOC_ENABLE_COMPRESSION_ZSTD 0
+#define MONGOC_ENABLE_COMPRESSION_ZSTD 1
 
 #if MONGOC_ENABLE_COMPRESSION_ZSTD != 1
 #  undef MONGOC_ENABLE_COMPRESSION_ZSTD
@@ -344,7 +360,7 @@
  * Set if we have the sched_getcpu() function for use with counters
  *
  */
-#define MONGOC_HAVE_SCHED_GETCPU 0
+#define MONGOC_HAVE_SCHED_GETCPU 1
 
 #if MONGOC_HAVE_SCHED_GETCPU != 1
 #  undef MONGOC_HAVE_SCHED_GETCPU
@@ -353,32 +369,56 @@
 /*
  * Set if tracing is enabled. Logs things like network communication and
  * entry/exit of certain functions.
- *
  */
 #define MONGOC_TRACE 1
 
-#if MONGOC_TRACE != 1
-#  undef MONGOC_TRACE
-#endif
-
 /*
- * Set if we have ICU support.
+ * Set if we have Client Side Encryption support.
  */
-#define MONGOC_ENABLE_ICU 0
 
-#if MONGOC_ENABLE_ICU != 1
-#  undef MONGOC_ENABLE_ICU
+#define MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION 1
+
+#if MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION != 1
+#  undef MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION
 #endif
 
 
 /*
- * NOTICE:
- * If you're about to update this file and add a config flag, make sure to
- * update:
- * o The bitfield in mongoc-handshake-private.h
- * o _get_config_bitfield() in mongoc-handshake.c
- * o examples/parse_handshake_cfg.py
- * o test_handshake_config_string in test-mongoc-handshake.c
+ * Set if struct sockaddr_storage has __ss_family (instead of ss_family)
  */
 
+#define MONGOC_HAVE_SS_FAMILY 1
+
+#if MONGOC_HAVE_SS_FAMILY != 1
+#  undef MONGOC_HAVE_SS_FAMILY
+#endif
+
+/*
+ * Set if building with AWS IAM support.
+ */
+#define MONGOC_ENABLE_MONGODB_AWS_AUTH 1
+
+#if MONGOC_ENABLE_MONGODB_AWS_AUTH != 1
+#  undef MONGOC_ENABLE_MONGODB_AWS_AUTH
+#endif
+
+enum {
+   /**
+    * @brief Compile-time constant determining whether the mongoc library was
+    * compiled with tracing enabled.
+    *
+    * Can be controlled with the “ENABLE_TRACING” configure-time boolean option
+    */
+   MONGOC_TRACE_ENABLED = MONGOC_TRACE,
+   /**
+    * @brief Compile-time constant indicating whether the mongoc library was
+    * compiled with SRV server discovery support.
+    *
+    * Can be controled with the “ENABLE_SRV” configure-time boolean option.
+    */
+   MONGOC_SRV_ENABLED = MONGOC_ENABLE_SRV,
+};
+
+/* clang-format on */
+
 #endif /* MONGOC_CONFIG_H */
