@@ -14,12 +14,12 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 #ifndef MONGOC_HANDSHAKE_COMPILER_PRIVATE_H
 #define MONGOC_HANDSHAKE_COMPILER_PRIVATE_H
 
-#include "mongoc/mongoc-config.h"
-#include "mongoc/mongoc-util-private.h"
+#include "mongoc-config.h"
+#include "mongoc-util-private.h"
 
 /*
  * Thanks to:
@@ -46,10 +46,9 @@
 #define MONGOC_COMPILER_VERSION MONGOC_EVALUATE_STR (_MSC_VER)
 #elif defined(__PGI)
 #define MONGOC_COMPILER "Portland PGCC"
-#define MONGOC_COMPILER_VERSION                                     \
-   MONGOC_EVALUATE_STR (__PGIC__)                                   \
-   "." MONGOC_EVALUATE_STR (__PGIC_MINOR) "." MONGOC_EVALUATE_STR ( \
-      __PGIC_PATCHLEVEL__)
+#define MONGOC_COMPILER_VERSION   \
+   MONGOC_EVALUATE_STR (__PGIC__) \
+   "." MONGOC_EVALUATE_STR (__PGIC_MINOR) "." MONGOC_EVALUATE_STR (__PGIC_PATCHLEVEL__)
 #elif defined(__SUNPRO_C) || defined(__SUNPRO_CC)
 #define MONGOC_COMPILER "Solaris Studio"
 #define MONGOC_COMPILER_VERSION MONGOC_EVALUATE_STR (__SUNPRO_C)
