@@ -1,7 +1,7 @@
 /*
  * Copyright 2015 MongoDB Inc.
- *
  * Licensed under the Apache License, Version 2.0 (the "License");
+ *
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
@@ -14,10 +14,12 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-host-list-private.h"
+#include <inttypes.h> // PRIu16
+
+#include "mongoc-host-list-private.h"
 /* strcasecmp on windows */
-#include "mongoc/mongoc-util-private.h"
-#include "mongoc/utlist.h"
+#include "mongoc-util-private.h"
+#include "utlist.h"
 
 /*
  *--------------------------------------------------------------------------
@@ -32,10 +34,7 @@
  *--------------------------------------------------------------------------
  */
 mongoc_host_list_t *
-_mongoc_host_list_push (const char *host,
-                        uint16_t port,
-                        int family,
-                        mongoc_host_list_t *next)
+_mongoc_host_list_push (const char *host, uint16_t port, int family, mongoc_host_list_t *next)
 {
    mongoc_host_list_t *h;
 
@@ -44,8 +43,7 @@ _mongoc_host_list_push (const char *host,
    h = bson_malloc0 (sizeof (mongoc_host_list_t));
    bson_strncpy (h->host, host, sizeof h->host);
    h->port = port;
-   bson_snprintf (
-      h->host_and_port, sizeof h->host_and_port, "%s:%hu", host, port);
+   bson_snprintf (h->host_and_port, sizeof h->host_and_port, "%s:%hu", host, port);
 
    h->family = family;
    h->next = next;
@@ -53,33 +51,100 @@ _mongoc_host_list_push (const char *host,
    return h;
 }
 
-/* Duplicates the elements of {src}, creating a new chain,
- * optionally prepended to an existing chain {next}.
- *
- * Note that as a side-effect of the implementation,
- * this reverses the order of src's copy in the destination.
- */
-mongoc_host_list_t *
-_mongoc_host_list_copy (const mongoc_host_list_t *src, mongoc_host_list_t *next)
+static mongoc_host_list_t *
+_mongoc_host_list_find_host_and_port (mongoc_host_list_t *hosts, const char *host_and_port)
 {
-   mongoc_host_list_t *h = NULL;
-   const mongoc_host_list_t *src_iter;
-
-   LL_FOREACH (src, src_iter)
+   mongoc_host_list_t *iter;
+   LL_FOREACH (hosts, iter)
    {
-      h = bson_malloc0 (sizeof (mongoc_host_list_t));
-      memcpy (h, src_iter, sizeof (mongoc_host_list_t));
+      BSON_ASSERT (iter);
 
-      LL_PREPEND (next, h);
+      if (strcasecmp (iter->host_and_port, host_and_port) == 0) {
+         return iter;
+      }
    }
 
-   return h;
+   return NULL;
 }
 
 /*
  *--------------------------------------------------------------------------
  *
- * _mongoc_host_list_equal --
+ * _mongoc_host_list_upsert --
+ *
+ *       If new_host is not already in list, copy and add it to the end of list.
+ *       If *list == NULL, then it will be set to a new host.
+ *
+ * Returns:
+ *       Nothing.
+ *
+ *--------------------------------------------------------------------------
+ */
+void
+_mongoc_host_list_upsert (mongoc_host_list_t **list, const mongoc_host_list_t *new_host)
+{
+   mongoc_host_list_t *link = NULL;
+   mongoc_host_list_t *next_link = NULL;
+
+   BSON_ASSERT (list);
+   if (!new_host) {
+      return;
+   }
+
+   link = _mongoc_host_list_find_host_and_port (*list, new_host->host_and_port);
+
+   if (!link) {
+      link = bson_malloc0 (sizeof (mongoc_host_list_t));
+      LL_APPEND (*list, link);
+   } else {
+      /* Make sure linking is preserved when copying data into final. */
+      next_link = link->next;
+   }
+
+   memcpy (link, new_host, sizeof (mongoc_host_list_t));
+   link->next = next_link;
+}
+
+
+/* Duplicates a host list.
+ */
+mongoc_host_list_t *
+_mongoc_host_list_copy_all (const mongoc_host_list_t *src)
+{
+   mongoc_host_list_t *tail = NULL;
+   const mongoc_host_list_t *src_iter;
+   mongoc_host_list_t *head = NULL;
+
+   LL_FOREACH (src, src_iter)
+   {
+      tail = bson_malloc0 (sizeof (mongoc_host_list_t));
+      memcpy (tail, src_iter, sizeof (mongoc_host_list_t));
+
+      LL_PREPEND (head, tail);
+   }
+
+   return head;
+}
+
+size_t
+_mongoc_host_list_length (const mongoc_host_list_t *list)
+{
+   const mongoc_host_list_t *tmp;
+   size_t counter = 0u;
+
+   tmp = list;
+   while (tmp) {
+      tmp = tmp->next;
+      counter++;
+   }
+
+   return counter;
+}
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * _mongoc_host_list_compare_one --
  *
  *       Check two hosts have the same domain (case-insensitive), port,
  *       and address family.
@@ -90,11 +155,15 @@ _mongoc_host_list_copy (const mongoc_host_list_t *src, mongoc_host_list_t *next)
  *--------------------------------------------------------------------------
  */
 bool
-_mongoc_host_list_equal (const mongoc_host_list_t *host_a,
-                         const mongoc_host_list_t *host_b)
+_mongoc_host_list_compare_one (const mongoc_host_list_t *host_a, const mongoc_host_list_t *host_b)
 {
-   return (!strcasecmp (host_a->host_and_port, host_b->host_and_port) &&
-           host_a->family == host_b->family);
+   return (0 == strcasecmp (host_a->host_and_port, host_b->host_and_port) && host_a->family == host_b->family);
+}
+
+bool
+_mongoc_host_list_contains_one (mongoc_host_list_t *host_list, mongoc_host_list_t *host)
+{
+   return NULL != _mongoc_host_list_find_host_and_port (host_list, host->host_and_port);
 }
 
 
@@ -141,9 +210,7 @@ _mongoc_host_list_from_string (mongoc_host_list_t *link_, const char *address)
 }
 
 bool
-_mongoc_host_list_from_string_with_err (mongoc_host_list_t *link_,
-                                        const char *address,
-                                        bson_error_t *error)
+_mongoc_host_list_from_string_with_err (mongoc_host_list_t *link_, const char *address, bson_error_t *error)
 {
    char *close_bracket;
    char *sport;
@@ -159,15 +226,27 @@ _mongoc_host_list_from_string_with_err (mongoc_host_list_t *link_,
       /* if present, the port should immediately follow after ] */
       sport = strchr (close_bracket, ':');
       if (sport > close_bracket + 1) {
+         bson_set_error (error,
+                         MONGOC_ERROR_COMMAND,
+                         MONGOC_ERROR_COMMAND_INVALID_ARG,
+                         "If present, port should immediately follow the \"]\""
+                         "in an IPv6 address");
          return false;
       }
 
       /* otherwise ] should be the last char. */
       if (!sport && *(close_bracket + 1) != '\0') {
+         bson_set_error (error,
+                         MONGOC_ERROR_COMMAND,
+                         MONGOC_ERROR_COMMAND_INVALID_ARG,
+                         "If port is not supplied, \"[\" should be the last"
+                         "character");
          return false;
       }
 
       if (*address != '[') {
+         bson_set_error (
+            error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_COMMAND_INVALID_ARG, "Missing matching bracket \"[\"");
          return false;
       }
 
@@ -182,10 +261,15 @@ _mongoc_host_list_from_string_with_err (mongoc_host_list_t *link_,
    if (sport) {
       if (sport == address) {
          /* bad address like ":27017" */
+         bson_set_error (error,
+                         MONGOC_ERROR_COMMAND,
+                         MONGOC_ERROR_COMMAND_INVALID_ARG,
+                         "Bad address, \":\" should not be first character");
          return false;
       }
 
       if (!mongoc_parse_port (&port, sport + 1)) {
+         bson_set_error (error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_COMMAND_INVALID_ARG, "Port could not be parsed");
          return false;
       }
 
@@ -222,10 +306,7 @@ _mongoc_host_list_from_hostport_with_err (mongoc_host_list_t *link_,
    link_->port = port;
 
    if (host_len == 0) {
-      bson_set_error (error,
-                      MONGOC_ERROR_STREAM,
-                      MONGOC_ERROR_STREAM_NAME_RESOLUTION,
-                      "Empty hostname in URI");
+      bson_set_error (error, MONGOC_ERROR_STREAM, MONGOC_ERROR_STREAM_NAME_RESOLUTION, "Empty hostname in URI");
       return false;
    }
 
@@ -244,13 +325,23 @@ _mongoc_host_list_from_hostport_with_err (mongoc_host_list_t *link_,
    if (strchr (host, ':')) {
       link_->family = AF_INET6;
 
-      mongoc_lowercase (link_->host, link_->host);
-      bson_snprintf (link_->host_and_port,
-                     sizeof link_->host_and_port,
-                     "[%s]:%hu",
-                     link_->host,
-                     link_->port);
+      // Check that IPv6 literal is two less than the max to account for `[` and
+      // `]` added below.
+      if (host_len > BSON_HOST_NAME_MAX - 2) {
+         bson_set_error (error,
+                         MONGOC_ERROR_STREAM,
+                         MONGOC_ERROR_STREAM_NAME_RESOLUTION,
+                         "IPv6 literal provided in URI is too long, max is %d chars",
+                         BSON_HOST_NAME_MAX - 2);
+         return false;
+      }
 
+      mongoc_lowercase (link_->host, link_->host);
+      int req =
+         bson_snprintf (link_->host_and_port, sizeof link_->host_and_port, "[%s]:%" PRIu16, link_->host, link_->port);
+      BSON_ASSERT (bson_in_range_size_t_signed (req));
+      // Use `<`, not `<=` to account for NULL byte.
+      BSON_ASSERT ((size_t) req < sizeof link_->host_and_port);
    } else if (strchr (host, '/') && strstr (host, ".sock")) {
       link_->family = AF_UNIX;
       bson_strncpy (link_->host_and_port, link_->host, host_len + 1);
@@ -259,11 +350,11 @@ _mongoc_host_list_from_hostport_with_err (mongoc_host_list_t *link_,
       link_->family = AF_UNSPEC;
 
       mongoc_lowercase (link_->host, link_->host);
-      bson_snprintf (link_->host_and_port,
-                     sizeof link_->host_and_port,
-                     "%s:%hu",
-                     link_->host,
-                     link_->port);
+      int req =
+         bson_snprintf (link_->host_and_port, sizeof link_->host_and_port, "%s:%" PRIu16, link_->host, link_->port);
+      BSON_ASSERT (bson_in_range_size_t_signed (req));
+      // Use `<`, not `<=` to account for NULL byte.
+      BSON_ASSERT ((size_t) req < sizeof link_->host_and_port);
    }
 
    link_->next = NULL;
@@ -271,9 +362,7 @@ _mongoc_host_list_from_hostport_with_err (mongoc_host_list_t *link_,
 }
 
 void
-_mongoc_host_list_remove_host (mongoc_host_list_t **hosts,
-                               const char *host,
-                               uint16_t port)
+_mongoc_host_list_remove_host (mongoc_host_list_t **hosts, const char *host, uint16_t port)
 {
    mongoc_host_list_t *current;
    mongoc_host_list_t *prev = NULL;
@@ -291,4 +380,4 @@ _mongoc_host_list_remove_host (mongoc_host_list_t **hosts,
          break;
       }
    }
-}
\ No newline at end of file
+}
