@@ -14,14 +14,12 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_LOG_PRIVATE_H
 #define MONGOC_LOG_PRIVATE_H
 
-#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
-#error "Only <mongoc/mongoc.h> can be included directly."
-#endif
-
-#include "mongoc/mongoc-iovec.h"
+#include "mongoc-iovec.h"
 
 /* just for testing */
 void
@@ -34,8 +32,6 @@ void
 mongoc_log_trace_bytes (const char *domain, const uint8_t *_b, size_t _l);
 
 void
-mongoc_log_trace_iovec (const char *domain,
-                        const mongoc_iovec_t *_iov,
-                        size_t _iovcnt);
+mongoc_log_trace_iovec (const char *domain, const mongoc_iovec_t *_iov, size_t _iovcnt);
 
 #endif /* MONGOC_LOG_PRIVATE_H */
