@@ -14,27 +14,23 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-opts-helpers-private.h"
-#include "mongoc/mongoc-client-session-private.h"
-#include "mongoc/mongoc-write-concern-private.h"
-#include "mongoc/mongoc-util-private.h"
-#include "mongoc/mongoc-read-concern-private.h"
+#include "mongoc-opts-helpers-private.h"
+#include "mongoc-client-session-private.h"
+#include "mongoc-write-concern-private.h"
+#include "mongoc-util-private.h"
+#include "mongoc-read-concern-private.h"
 
-#define BSON_ERR(...)                                                       \
-   do {                                                                     \
-      bson_set_error (                                                      \
-         error, MONGOC_ERROR_BSON, MONGOC_ERROR_BSON_INVALID, __VA_ARGS__); \
-      return false;                                                         \
+#define BSON_ERR(...)                                                                    \
+   do {                                                                                  \
+      bson_set_error (error, MONGOC_ERROR_BSON, MONGOC_ERROR_BSON_INVALID, __VA_ARGS__); \
+      return false;                                                                      \
    } while (0)
 
 
-#define CONVERSION_ERR(...)                             \
-   do {                                                 \
-      bson_set_error (error,                            \
-                      MONGOC_ERROR_COMMAND,             \
-                      MONGOC_ERROR_COMMAND_INVALID_ARG, \
-                      __VA_ARGS__);                     \
-      return false;                                     \
+#define CONVERSION_ERR(...)                                                                        \
+   do {                                                                                            \
+      bson_set_error (error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_COMMAND_INVALID_ARG, __VA_ARGS__); \
+      return false;                                                                                \
    } while (0)
 
 
@@ -52,19 +48,17 @@ _mongoc_timestamp_set (mongoc_timestamp_t *dst, mongoc_timestamp_t *src)
 }
 
 void
-_mongoc_timestamp_set_from_bson (mongoc_timestamp_t *timestamp,
-                                 bson_iter_t *iter)
+_mongoc_timestamp_set_from_bson (mongoc_timestamp_t *timestamp, bson_iter_t *iter)
 {
    bson_iter_timestamp (iter, &(timestamp->timestamp), &(timestamp->increment));
 }
 
 void
-_mongoc_timestamp_append (mongoc_timestamp_t *timestamp,
-                          bson_t *bson,
-                          char *key)
+_mongoc_timestamp_append (mongoc_timestamp_t *timestamp, bson_t *bson, char *key)
 {
-   bson_append_timestamp (
-      bson, key, strlen (key), timestamp->timestamp, timestamp->increment);
+   const size_t len = strlen (key);
+   BSON_ASSERT (bson_in_range_unsigned (int, len));
+   bson_append_timestamp (bson, key, (int) len, timestamp->timestamp, timestamp->increment);
 }
 
 void
@@ -75,15 +69,14 @@ _mongoc_timestamp_clear (mongoc_timestamp_t *timestamp)
 }
 
 bool
-_mongoc_convert_document (mongoc_client_t *client,
-                          const bson_iter_t *iter,
-                          bson_t *doc,
-                          bson_error_t *error)
+_mongoc_convert_document (mongoc_client_t *client, const bson_iter_t *iter, bson_t *doc, bson_error_t *error)
 {
    uint32_t len;
    const uint8_t *data;
    bson_t value;
 
+   BSON_UNUSED (client);
+
    if (!BSON_ITER_HOLDS_DOCUMENT (iter)) {
       CONVERSION_ERR ("Invalid field \"%s\" in opts, should contain document,"
                       " not %s",
@@ -103,15 +96,14 @@ _mongoc_convert_document (mongoc_client_t *client,
 }
 
 bool
-_mongoc_convert_array (mongoc_client_t *client,
-                       const bson_iter_t *iter,
-                       bson_t *doc,
-                       bson_error_t *error)
+_mongoc_convert_array (mongoc_client_t *client, const bson_iter_t *iter, bson_t *doc, bson_error_t *error)
 {
    uint32_t len;
    const uint8_t *data;
    bson_t value;
 
+   BSON_UNUSED (client);
+
    if (!BSON_ITER_HOLDS_ARRAY (iter)) {
       CONVERSION_ERR ("Invalid field \"%s\" in opts, should contain array,"
                       " not %s",
@@ -131,13 +123,12 @@ _mongoc_convert_array (mongoc_client_t *client,
 }
 
 bool
-_mongoc_convert_int64_positive (mongoc_client_t *client,
-                                const bson_iter_t *iter,
-                                int64_t *num,
-                                bson_error_t *error)
+_mongoc_convert_int64_positive (mongoc_client_t *client, const bson_iter_t *iter, int64_t *num, bson_error_t *error)
 {
    int64_t i;
 
+   BSON_UNUSED (client);
+
    if (!BSON_ITER_HOLDS_NUMBER (iter)) {
       CONVERSION_ERR ("Invalid field \"%s\" in opts, should contain number,"
                       " not %s",
@@ -158,23 +149,19 @@ _mongoc_convert_int64_positive (mongoc_client_t *client,
 }
 
 bool
-_mongoc_convert_int32_t (mongoc_client_t *client,
-                         const bson_iter_t *iter,
-                         int32_t *num,
-                         bson_error_t *error)
+_mongoc_convert_int32_t (mongoc_client_t *client, const bson_iter_t *iter, int32_t *num, bson_error_t *error)
 {
    int64_t i;
 
+   BSON_UNUSED (client);
+
    if (!BSON_ITER_HOLDS_NUMBER (iter)) {
       CONVERSION_ERR ("Invalid field \"%s\" in opts", bson_iter_key (iter));
    }
 
    i = bson_iter_as_int64 (iter);
    if (i > INT32_MAX || i < INT32_MIN) {
-      CONVERSION_ERR ("Invalid field \"%s\" in opts: %" PRId64
-                      " out of range for int32",
-                      bson_iter_key (iter),
-                      i);
+      CONVERSION_ERR ("Invalid field \"%s\" in opts: %" PRId64 " out of range for int32", bson_iter_key (iter), i);
    }
 
    *num = (int32_t) i;
@@ -183,10 +170,7 @@ _mongoc_convert_int32_t (mongoc_client_t *client,
 }
 
 bool
-_mongoc_convert_int32_positive (mongoc_client_t *client,
-                                const bson_iter_t *iter,
-                                int32_t *num,
-                                bson_error_t *error)
+_mongoc_convert_int32_positive (mongoc_client_t *client, const bson_iter_t *iter, int32_t *num, bson_error_t *error)
 {
    int32_t i;
 
@@ -195,10 +179,7 @@ _mongoc_convert_int32_positive (mongoc_client_t *client,
    }
 
    if (i <= 0) {
-      CONVERSION_ERR (
-         "Invalid field \"%s\" in opts, should be greater than 0, not %d",
-         bson_iter_key (iter),
-         i);
+      CONVERSION_ERR ("Invalid field \"%s\" in opts, should be greater than 0, not %d", bson_iter_key (iter), i);
    }
 
    *num = i;
@@ -207,11 +188,10 @@ _mongoc_convert_int32_positive (mongoc_client_t *client,
 }
 
 bool
-_mongoc_convert_bool (mongoc_client_t *client,
-                      const bson_iter_t *iter,
-                      bool *flag,
-                      bson_error_t *error)
+_mongoc_convert_bool (mongoc_client_t *client, const bson_iter_t *iter, bool *flag, bson_error_t *error)
 {
+   BSON_UNUSED (client);
+
    if (BSON_ITER_HOLDS_BOOL (iter)) {
       *flag = bson_iter_bool (iter);
       return true;
@@ -229,7 +209,11 @@ _mongoc_convert_bson_value_t (mongoc_client_t *client,
                               bson_value_t *value,
                               bson_error_t *error)
 {
+   BSON_UNUSED (client);
+   BSON_UNUSED (error);
+
    bson_value_copy (bson_iter_value ((bson_iter_t *) iter), value);
+
    return true;
 }
 
@@ -239,16 +223,19 @@ _mongoc_convert_timestamp (mongoc_client_t *client,
                            mongoc_timestamp_t *timestamp,
                            bson_error_t *error)
 {
+   BSON_UNUSED (client);
+   BSON_UNUSED (error);
+
    bson_iter_timestamp (iter, &timestamp->timestamp, &timestamp->increment);
+
    return true;
 }
 
 bool
-_mongoc_convert_utf8 (mongoc_client_t *client,
-                      const bson_iter_t *iter,
-                      const char **str,
-                      bson_error_t *error)
+_mongoc_convert_utf8 (mongoc_client_t *client, const bson_iter_t *iter, const char **str, bson_error_t *error)
 {
+   BSON_UNUSED (client);
+
    if (BSON_ITER_HOLDS_UTF8 (iter)) {
       *str = bson_iter_utf8 (iter, NULL);
       return true;
@@ -266,6 +253,8 @@ _mongoc_convert_validate_flags (mongoc_client_t *client,
                                 bson_validate_flags_t *flags,
                                 bson_error_t *error)
 {
+   BSON_UNUSED (client);
+
    if (BSON_ITER_HOLDS_BOOL (iter)) {
       if (!bson_iter_as_bool (iter)) {
          *flags = BSON_VALIDATE_NONE;
@@ -302,6 +291,8 @@ _mongoc_convert_write_concern (mongoc_client_t *client,
 {
    mongoc_write_concern_t *tmp;
 
+   BSON_UNUSED (client);
+
    tmp = _mongoc_write_concern_new_from_iter (iter, error);
    if (tmp) {
       *wc = tmp;
@@ -312,13 +303,12 @@ _mongoc_convert_write_concern (mongoc_client_t *client,
 }
 
 bool
-_mongoc_convert_server_id (mongoc_client_t *client,
-                           const bson_iter_t *iter,
-                           uint32_t *server_id,
-                           bson_error_t *error)
+_mongoc_convert_server_id (mongoc_client_t *client, const bson_iter_t *iter, uint32_t *server_id, bson_error_t *error)
 {
    int64_t tmp;
 
+   BSON_UNUSED (client);
+
    if (!BSON_ITER_HOLDS_INT (iter)) {
       CONVERSION_ERR ("The serverId option must be an integer");
    }
@@ -338,9 +328,24 @@ _mongoc_convert_read_concern (mongoc_client_t *client,
                               mongoc_read_concern_t **rc,
                               bson_error_t *error)
 {
+   BSON_UNUSED (client);
+
    *rc = _mongoc_read_concern_new_from_iter (iter, error);
    if (!*rc) {
       return false;
    }
    return true;
 }
+
+bool
+_mongoc_convert_hint (mongoc_client_t *client, const bson_iter_t *iter, bson_value_t *value, bson_error_t *error)
+{
+   BSON_UNUSED (client);
+
+   if (BSON_ITER_HOLDS_UTF8 (iter) || BSON_ITER_HOLDS_DOCUMENT (iter)) {
+      bson_value_copy (bson_iter_value ((bson_iter_t *) iter), value);
+      return true;
+   }
+
+   CONVERSION_ERR ("The hint option must be a string or document");
+}
