@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_STREAM_TLS_SECURE_CHANNEL_H
 #define MONGOC_STREAM_TLS_SECURE_CHANNEL_H
@@ -22,15 +22,13 @@
 #ifdef MONGOC_ENABLE_SSL_SECURE_CHANNEL
 #include <bson/bson.h>
 
-#include "mongoc/mongoc-macros.h"
+#include "mongoc-macros.h"
 
 BSON_BEGIN_DECLS
 
 MONGOC_EXPORT (mongoc_stream_t *)
-mongoc_stream_tls_secure_channel_new (mongoc_stream_t *base_stream,
-                                      const char *host,
-                                      mongoc_ssl_opt_t *opt,
-                                      int client);
+mongoc_stream_tls_secure_channel_new (mongoc_stream_t *base_stream, const char *host, mongoc_ssl_opt_t *opt, int client)
+   BSON_GNUC_WARN_UNUSED_RESULT;
 
 BSON_END_DECLS
 
